#include "defines.h"
#include "keys.h"
#include "tilestuf.h"
#include "murn.h"

const BIG_TILE_WIDTH 	= 	32	;
const BIG_TILE_HEIGHT 	= 	32	;
const BIG_TILE_SHIFT 	= 	5	;

const NUM_FRED_FRAMES	= 	4   ;
const NUM_ME_FRAMES	= 	16   ;

const NUM_BACK_TILE_FRAMES	=  20  ;
const NUM_FRONT_TILE_FRAMES	= 	14  ;

const TILE_WIDTH 	= 	16	;
const TILE_HEIGHT 	= 	16	;
const TILE_SHIFT 	= 	4	;

#define NUM_BADDIES		10

#define ALIVE 	1
#define DEAD 	0

#define BOUNCER 0


extern char volatile Keyboard[128];
extern char volatile KeyScan;    // modified by keyboard interrupt handler

void CleanUp(struct Picture_Type **my_pics,
			 struct Picture_Type **springs,
//			 struct Picture_Type **tiles,
			 struct Picture_Type **back_tiles,
			 struct Picture_Type *map,
			 struct Picture_Type *back_map)
{
	printf("CleanUp(my_pics)\n");
	FreeTiles(my_pics,NUM_ME_FRAMES);

//	printf("CleanUp(tiles)\n");
//	FreeTiles(tiles,NUM_FRONT_TILE_FRAMES);

	printf("CleanUp(back_tiles)\n");
	FreeTiles(back_tiles,NUM_BACK_TILE_FRAMES);

	printf("CleanUp(springs)\n");
	FreeTiles(springs,NUM_FRED_FRAMES);
	printf("CleanUp(map)\n");
	FreePic(map);

	printf("CleanUp(back map)\n");
	FreePic(back_map);

	printf("Done\n");
	getch();
	ScreenMode(0x03);
}

void DrawMap(unsigned int px,
			unsigned int py,
			struct Picture_Type *map,
			struct Picture_Type **tiles,
			uchar far * vscreen)
{
	register int mapx,
		mapy=0,
		tilex,
		screenx=0,
		screeny = 16,
		map_tmp=0,
		screen_tmp=0,tmp=0;

	mapx = map_tmp = px >> TILE_SHIFT ;
	mapy = py >> TILE_SHIFT;
	tilex = px % TILE_WIDTH;
	screenx = screen_tmp= TILE_WIDTH-tilex;

	while ( screeny <= 160 ) {
		while ( screenx < 304 ) {
			tmp=*(map->Data+(mapy*map->Width)+mapx);
			if (tmp!=0) {
				DrawFullTile(screenx,screeny,
							 tiles[tmp],
							 vscreen);
			}

			mapx++;
			screenx += TILE_WIDTH;
		}

		mapx=map_tmp;
		screenx=screen_tmp;
		screeny+=TILE_HEIGHT;
		mapy++;
	}
}


void DrawNewMap(unsigned int px,
			unsigned int py,
			struct Picture_Type *map,
			struct Picture_Type **tiles,
			uchar far * vscreen)
{
	register int mapx,
		mapy=0,
		tilex,
		tiley,
		screenx = 0,
		screeny = 0,
		map_tmp=0,
		tmp=0,
		amountx =0,
		amounty =0,
		amountx_tmp=0,
		tilex_tmp=0;

	mapx = map_tmp = px >> BIG_TILE_SHIFT;
	mapy = py >> BIG_TILE_SHIFT;

	tilex = tilex_tmp = px % BIG_TILE_WIDTH;
	tiley = py % BIG_TILE_HEIGHT;

	amountx = amountx_tmp = BIG_TILE_WIDTH-tilex;
	amounty = BIG_TILE_HEIGHT-tiley;

	while ( screeny < SCREEN_BOTTOM ) {
		while ( screenx < SCREEN_RIGHT ) {
			tmp = *(map->Data+(mapy*map->Width)+mapx);

			DrawPieceOfTile(screenx,screeny,
							 tilex,tiley,
							 amountx,amounty,
							 tiles[tmp],
							 vscreen);
			mapx++;
			tilex=0;
			screenx += amountx;
			amountx=tiles[0]->Width;
			if ((screenx+amountx) > SCREEN_RIGHT) {
				amountx = SCREEN_RIGHT - screenx;
			}
		}

		mapx = map_tmp;
		mapy++;

		amountx = amountx_tmp;
		tilex = tilex_tmp;
		tiley=0;

		screenx = 0;
		screeny += amounty;
		amounty = tiles[0]->Height;
		if ((screeny+amounty) > SCREEN_BOTTOM) {
			amounty = SCREEN_BOTTOM - screeny;
		}
	}
}

void DrawBaddies(unsigned int px,
				unsigned int py,
				struct Picture_Type **baddie_pics,
				struct Baddie bad[NUM_BADDIES],
				uchar far * vscreen)
{
	int	i=0;

	for (i=0;i<NUM_BADDIES;i++) {
		if ((bad[i].x-px > 0) &&
			(bad[i].x-px < 304) &&
			(bad[i].y-py > 0) &&
			(bad[i].y-py < 184)){
			DrawTransTile(bad[i].x-px,bad[i].y-py,
						 baddie_pics[bad[i].frame],
						 vscreen);
		}
	}

}

void DoBouncer(struct Baddie *bad,struct Picture_Type *map)
{
	if (bad->status==ALIVE) {
		if (bad->dir==LEFT) {
			if ((*(map->Data + ((bad->y >> BIG_TILE_SHIFT) * map->Width) + ((bad->x + bad->x_inc) >> BIG_TILE_SHIFT ))) > 3) {
				if ((*(map->Data + (((bad->y+15) >> BIG_TILE_SHIFT) * map->Width) + ((bad->x + bad->x_inc) >> BIG_TILE_SHIFT ))) > 3) {
						bad->x-=bad->x_inc;
				}
			} else {
				bad->dir=RIGHT;
			}
		} else {
			bad->dir=RIGHT;
		}

		if (bad->dir==RIGHT) {
			if (((*(map->Data + ((bad->y >> BIG_TILE_SHIFT) * map->Width) + ((bad->x + 16 + bad->x_inc) >> BIG_TILE_SHIFT )) > 3)
				&& (*(map->Data + (((bad->y+15) >> BIG_TILE_SHIFT) * map->Width) + ((bad->x + 16 + bad->x_inc) >> BIG_TILE_SHIFT )) > 3))) {
					bad->x+=bad->x_inc;
			} else {
				bad->dir=LEFT;
			}
		}

		// change frame no matter what
		bad->frame++;
		if (bad->frame==4) {
			bad->frame=0;
		}
	}

}

void ManageBaddies(struct Baddie bad[NUM_BADDIES],
					struct Picture_Type *map)
{
	int i;

	for (i=0;i < NUM_BADDIES;i++) {
		switch (bad[i].type) {
			case BOUNCER: DoBouncer(&bad[i],map);
							break;
			default : break;
		}
	}
}

void MoveMap(struct Sprite me,
			int *px,
			int *py,
			struct Picture_Type *map)
{

	if ( (me.x-160) > 0) {       // me in the left of the map
		*px=me.x-160;
		if ((*px) > ((map->Width << BIG_TILE_SHIFT)-SCREEN_RIGHT)) {
			*px=(map->Width << BIG_TILE_SHIFT)-SCREEN_RIGHT;
		}
	} else {
		*px=0;
	}

	if ( (me.y- 100) > 0) {       // me in the left of the map
		*py=me.y-100;
		if ((*py) > ((map->Height << BIG_TILE_SHIFT)-SCREEN_BOTTOM)) {
			*py=(map->Height << BIG_TILE_SHIFT)-SCREEN_BOTTOM;
		}
	} else {
		*py=0;
	}
}

int LoadSprite(char *filename,
				int number,
				struct Picture_Type ***sprite)
{
	int i;
	struct Picture_Type **Array;
	*sprite=(struct Picture_Type **)malloc(number*(sizeof(struct Picture_Type *)));
	Array=*sprite;
	for (i=0;i<number;i++)
	{
		filename[8]=(i/10)+48;
		filename[9]=(i%10)+48;

		printf("Loading %s..",filename);

		Array[i]=Load_PCX(filename,0);

		if (Array[i]==NULL) {
			printf("...Failed\n");
			getch();
			return(0);
		}
		printf(".Good\n");
	}

	return(1);
}

struct Baddie *LoadBaddies(void)
{
	struct Baddie *tmp;
	struct Picture_Type *baddie_map;
	int i=0,j=0,k=0;

	tmp=(struct Baddie *)malloc(NUM_BADDIES*(sizeof(struct Baddie )));

	if (tmp!=NULL) {
		baddie_map=Load_PCX("pcx\\baddies.pcc",0);
		if (baddie_map) {
			for (i=0;i<baddie_map->Height;i++) {
				for (j=0;j<baddie_map->Width;j++) {

					if ((*(baddie_map->Data+(i*baddie_map->Width)+j)) > 0) {
						if (k<NUM_BADDIES) {
							tmp[k].type = BOUNCER;
							tmp[k].x=j*32;
							tmp[k].y=i*32;
							tmp[k].x_inc=1;
							tmp[k].y_inc=1;
							tmp[k].frame=0;
							tmp[k].dir=RIGHT;
							tmp[k].status=ALIVE;
							printf("Baddie %d at %d->%d, %d->%d\n",k,j,tmp[k].x,i,tmp[k].y);
							k++;

						}
					}
				}
			}
			getch();
			printf("Done !\n");
			FreePic(baddie_map);
			return (tmp);
		}
	}
	return(NULL);
}

void main()
{

	int i;


	uchar far *screen;
	uchar far *vscreen;

//	struct Picture_Type **tiles;
	struct Picture_Type **back_tiles;
	struct Picture_Type **spring_pics;
	struct Picture_Type **me_pics;

	struct Picture_Type *map=NULL;
	struct Picture_Type *back_map=NULL;

	struct Baddie *baddies=NULL;
	struct Sprite me_sprite={0,100,100,WAITING,WAITING,1,0,0,4,1};
		/*	char frame;
			int x;
			int y;
			char status;
			char dir;
			char frame_inc;
			char speed;
			char counter;
			char jump_inc;
			char jump_counter;
		*/


	int posx=0,posy=0;
	int good=0;

	/*****************************************/

	ScreenMode(0x13);

	back_map=Load_PCX("pcx\\map01.pcc",0);

	baddies=LoadBaddies();

	good=LoadSprite("pcx\\tailXX.pcc",NUM_BACK_TILE_FRAMES,&back_tiles);
	if (good) {
		good=LoadSprite("pcx\\spriXX.pcc",NUM_FRED_FRAMES,&spring_pics);
		if (good==0) {
			ScreenMode(0x03);
			printf("LoadTiles Failed\n");
			exit(1);
		}
		good=LoadSprite("pcx\\murnXX.pcc",NUM_ME_FRAMES,&me_pics);
		if (good==0) {
			ScreenMode(0x03);
			printf("LoadTiles Failed\n");
			exit(1);
		}
	} else {
		if (good==0) {
			ScreenMode(0x03);
			printf("LoadTiles Failed\n");
			exit(1);
		}
	}

	LoadPaletteScreen("pcx\\tile00.pcc");

	screen=MK_FP(0xA000,0);

	if ((vscreen=(uchar far *)malloc(SCR_BLK))!=NULL)
	{
		InitKeyboard();

		do {
			memset(vscreen,0,SCR_BLK);

			DrawNewMap(posx,posy,back_map,back_tiles,vscreen);
//			DrawMap(posx<<1,posy<<1,map,tiles,vscreen);

			DrawBaddies(posx,posy,spring_pics,baddies,vscreen);
			DrawMe(me_sprite,posx,posy,me_pics,vscreen);

			ManageBaddies(baddies,map);
			DoKeys(&me_sprite);
			DoMe(&me_sprite,back_map);

			MoveMap(me_sprite,&posx,&posy,back_map);

			memcpy((uchar far* )screen,(uchar far* )vscreen,SCR_BLK);
		}
		while(!Keyboard[RETURN_PRESSED]);

		RestoreKeyboard();
		free(vscreen);
	}
	else {
		printf("Failed to malloc vscreen\n");
		getch();
	}

	CleanUp(me_pics,spring_pics,back_tiles,map,back_map);
   printf("Done.....(dos)\n");
}